@@ -1109,6 +1109,78 @@ TEST_CASE_METHOD(
   CHECK(!std::memcmp(data_c, data_r, data_r_size));
 }
 
+TEST_CASE_METHOD(
+    CSparseUnorderedWithDupsFx,
+    "Sparse unordered with dups reader: single tile query continuation",
+    "[sparse-unordered-with-dups][single-tile][continuation]") {
+  // Create default array.
+  reset_config();
+  create_default_array_1d();
+
+  // Write a fragment.
+  int coords[] = {1, 2};
+  uint64_t coords_size = sizeof(coords);
+  int data[] = {1, 2};
+  uint64_t data_size = sizeof(data);
+  write_1d_fragment(coords, &coords_size, data, &data_size);
+
+  tiledb_array_t* array = nullptr;
+  tiledb_query_t* query = nullptr;
+
+  // Try to read.
+  int coords_r[1];  // only room for one cell.
+  int data_r[1];
+  uint64_t coords_r_size = sizeof(coords_r);
+  uint64_t data_r_size = sizeof(data_r);
+  auto rc = read(
+      false,
+      false,
+      coords_r,
+      &coords_r_size,
+      data_r,
+      &data_r_size,
+      &query,
+      &array);
+  CHECK(rc == TILEDB_OK);
+
+  // Check incomplete query status.
+  tiledb_query_status_t status;
+  tiledb_query_get_status(ctx_, query, &status);
+  CHECK(status == TILEDB_INCOMPLETE);
+
+  // Should only read one cell (1 values).
+  CHECK(4 == data_r_size);
+  CHECK(4 == coords_r_size);
+
+  int coords_c_1[] = {1};
+  int data_c_1[] = {1};
+  CHECK(!std::memcmp(coords_c_1, coords_r, coords_r_size));
+  CHECK(!std::memcmp(data_c_1, data_r, data_r_size));
+
+  // Read again.
+  rc = tiledb_query_submit(ctx_, query);
+  CHECK(rc == TILEDB_OK);
+
+  // Check incomplete query status.
+  tiledb_query_get_status(ctx_, query, &status);
+  CHECK(status == TILEDB_COMPLETED);
+
+  // Should read last cell (1 values).
+  CHECK(4 == data_r_size);
+  CHECK(4 == coords_r_size);
+
+  int coords_c_2[] = {2};
+  int data_c_2[] = {2};
+  CHECK(!std::memcmp(coords_c_2, coords_r, coords_r_size));
+  CHECK(!std::memcmp(data_c_2, data_r, data_r_size));
+
+  // Clean up.
+  rc = tiledb_array_close(ctx_, array);
+  CHECK(rc == TILEDB_OK);
+  tiledb_array_free(&array);
+  tiledb_query_free(&query);
+}
+
 TEST_CASE_METHOD(
     CSparseUnorderedWithDupsVarDataFx,
     "Sparse unordered with dups reader: results shrinked due to data buffer",
@@ -1258,3 +1330,61 @@ TEST_CASE(
   CHECK(new_result_tiles_size == 1);
   CHECK(var_buffer_size == 6);
 }
+
+TEST_CASE(
+    "Sparse unordered with dups reader: test compute_var_size_offsets "
+    "continuation",
+    "[sparse-unordered-with-dups][compute_var_size_offsets][continuation]") {
+  // Make a vector of tiles.
+  std::vector<ResultTileWithBitmap<uint64_t>> rt = {make_tile(5)};
+
+  SECTION("- No bitmap") {
+  }
+
+  SECTION("- With bitmap") {
+    rt[0].bitmap_.resize(5, 1);
+  }
+
+  // Create the result_tiles pointer vector.
+  std::vector<ResultTile*> result_tiles(rt.size());
+  for (uint64_t i = 0; i < rt.size(); i++) {
+    result_tiles[i] = &rt[i];
+  }
+
+  // Create the cell_offsets vector.
+  std::vector<uint64_t> cell_offsets(rt.size() + 1);
+  uint64_t offset = 0;
+  for (uint64_t i = 0; i < rt.size(); i++) {
+    cell_offsets[i] = offset;
+    offset += rt[i].cell_num() - 2;
+  }
+  cell_offsets[rt.size()] = offset;
+
+  // Create a Query buffer.
+  tiledb::sm::QueryBuffer query_buffer;
+  uint64_t offsets[] = {2, 2, 2, 0, 0};
+  uint64_t offsets_size = sizeof(offsets);
+  query_buffer.buffer_ = offsets;
+  query_buffer.buffer_size_ = &offsets_size;
+  uint64_t var_size = 5;
+  query_buffer.buffer_var_size_ = &var_size;
+
+  // Call the function.
+  uint64_t new_result_tiles_size = result_tiles.size();
+  uint64_t var_buffer_size = 0;
+  REQUIRE(SparseUnorderedWithDupsReader<uint64_t>::compute_var_size_offsets<
+              uint64_t>(
+              &tiledb::test::g_helper_stats,
+              &result_tiles,
+              2,
+              &cell_offsets,
+              &query_buffer,
+              &new_result_tiles_size,
+              &var_buffer_size)
+              .ok());
+
+  // Validate results.
+  CHECK(cell_offsets[1] == 2);
+  CHECK(new_result_tiles_size == 1);
+  CHECK(var_buffer_size == 4);
+}
\ No newline at end of file
